/***************************************************************************
 *   Copyright (C) 2007 by Glad Deschrijver                                *
 *   Glad.Deschrijver@UGent.be                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QApplication>
#include <QButtonGroup>
#include <QColorDialog>
#include <QFontDialog>
#include <QGridLayout>
#include <QGroupBox>
#include <QHeaderView>
#include <QLineEdit>
#include <QPushButton>
#include <QRadioButton>
#include <QSettings>
#include <QTableWidget>
#include <QTextCharFormat>
#include <QVBoxLayout>

#include "appearancewidget.h"

AppearanceWidget::AppearanceWidget(QWidget *parent)
    : QWidget(parent)
{
	itemMargin = 10;

	standardAppearanceCheck = new QRadioButton(tr("&Standard"));
	standardAppearanceCheck->setWhatsThis("<p>" + tr("When this option "
	    "is checked, the default fonts and colors are used in "
	    "the output.") + "</p>");
	customAppearanceCheck = new QRadioButton(tr("&Custom"));
	customAppearanceCheck->setWhatsThis("<p>" + tr("When this option "
	    "is checked, the fonts and colors defined below will be used "
	    "in the output instead of the default ones.") + "</p>");

	QButtonGroup *buttonGroup = new QButtonGroup(this);
	buttonGroup->addButton(standardAppearanceCheck);
	buttonGroup->addButton(customAppearanceCheck);
	buttonGroup->setExclusive(true);
	connect(buttonGroup, SIGNAL(buttonClicked(QAbstractButton*)), this, SLOT(toggleCustom()));

	itemTable = new QTableWidget(this);
	QString itemWhatsThis = "<p>" + tr("Select the structure "
	    "appearing in the TikZ code for which you want to change "
	    "the fonts and colors.") + "</p>";
	itemTable->setWhatsThis(itemWhatsThis);
	itemTable->setShowGrid(true);
	itemTable->horizontalHeader()->hide();
	itemTable->verticalHeader()->hide();
	itemTable->setColumnCount(1);
	itemTable->setSelectionMode(QAbstractItemView::SingleSelection);
	itemTable->horizontalHeader()->setStretchLastSection(true);
	itemTable->setTabKeyNavigation(false);
	QPalette palette = itemTable->palette();
	QColor highlightBackgroundColor(QApplication::style()->standardPalette().color(QPalette::Normal, QPalette::AlternateBase));
	if (highlightBackgroundColor == QApplication::style()->standardPalette().color(QPalette::Normal, QPalette::Base))
		highlightBackgroundColor = highlightBackgroundColor.darker(110);
	palette.setColor(QPalette::Normal, QPalette::Highlight, highlightBackgroundColor);
	palette.setColor(QPalette::Normal, QPalette::HighlightedText, QPalette::Text);
	palette.setColor(QPalette::Inactive, QPalette::Highlight, highlightBackgroundColor);
	palette.setColor(QPalette::Inactive, QPalette::HighlightedText, QPalette::Text);
	itemTable->setPalette(palette);
	itemHighlighted = -1;
	connect(itemTable, SIGNAL(currentItemChanged(QTableWidgetItem*,QTableWidgetItem*)), this, SLOT(setItemHighlighted(QTableWidgetItem*)));

	fontButton = new QPushButton(tr("Change fo&nt"));
	QString fontWhatsThis = "<p>" + tr("Select the font in which "
	    "the structure that you selected in the box above should "
	    "be displayed.") + "</p>";
	fontButton->setWhatsThis(fontWhatsThis);
	fontButton->setEnabled(false);
	connect(fontButton, SIGNAL(clicked()), this, SLOT(showFontDialog()));

	colorButton = new QPushButton(tr("Change co&lor"));
	QString colorWhatsThis = "<p>" + tr("Select the text color in "
	    "which the structure that you selected in the box above "
	    "should be displayed.") + "</p>";
	colorButton->setWhatsThis(colorWhatsThis);
	colorButton->setEnabled(false);
	connect(colorButton, SIGNAL(clicked()), this, SLOT(showColorDialog()));

	appearanceWidget = new QGroupBox(tr("Change st&yle for") + ":");
	QGridLayout *appearanceLayout = new QGridLayout(appearanceWidget);
	appearanceLayout->addWidget(itemTable, 0, 0, 1, 3);
	appearanceLayout->addWidget(fontButton, 1, 1);
	appearanceLayout->addWidget(colorButton, 1, 2);

	QVBoxLayout *layout = new QVBoxLayout(this);
	layout->addWidget(standardAppearanceCheck);
	layout->addWidget(customAppearanceCheck);
	layout->addWidget(appearanceWidget);
	layout->setMargin(0);
}

void AppearanceWidget::setItemHighlighted(QTableWidgetItem *item)
{
	itemHighlighted = itemTable->row(item);
	setHighlightedForeground(itemColors.at(itemHighlighted));
	fontButton->setEnabled(true);
	colorButton->setEnabled(true);
}

void AppearanceWidget::setItemToolTip(QTableWidgetItem *item, const QFont &font)
{
	QFontMetrics metrics(font);
	itemTable->setRowHeight(itemHighlighted, metrics.height() + itemMargin);
	if (metrics.width(item->text()) >= itemTable->contentsRect().width() - 30)
		item->setToolTip(item->text());
	else
		item->setToolTip("");
}

void AppearanceWidget::showEvent(QShowEvent*)
{
	for (int i = 0; i < itemTable->rowCount(); ++i)
	{
		QFont font;
		font.fromString(itemFonts.at(i));
		setItemToolTip(itemTable->item(i, 0), font);
	}

	if (itemHighlighted < 0) return;
	setHighlightedForeground(itemColors.at(itemHighlighted));
}

void AppearanceWidget::addItem(const QString &titleName)
{
	int itemTableRowNum = itemTable->rowCount();
	itemTable->setRowCount(itemTableRowNum+1);

	QTableWidgetItem *item = new QTableWidgetItem;
	item->setText(titleName);
	item->setTextAlignment(Qt::AlignLeft | Qt::AlignVCenter);
	item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
	itemTable->setItem(itemTableRowNum, 0, item);
}

void AppearanceWidget::addItemFont(const QString &fontName)
{
	int rowNum = itemFonts.size();
	itemFonts.append(fontName);

	QFont font;
	font.fromString(fontName);
	itemTable->item(rowNum, 0)->setFont(font);

	QFontMetrics metrics(font);
	itemTable->setRowHeight(rowNum, metrics.height() + itemMargin);
}

void AppearanceWidget::addItemColor(const QString &colorName)
{
	int rowNum = itemFonts.size();
	itemColors.append(colorName);
	QColor color(colorName);
	itemTable->item(rowNum, 0)->setForeground(color);
}

void AppearanceWidget::addItems(const QStringList &titleNames)
{
	for (int i = 0; i < titleNames.size(); ++i)
		addItem(titleNames.at(i));
}

void AppearanceWidget::setItemFonts(const QStringList &fontNames)
{
	itemFonts = fontNames;
	for (int i = 0; i < itemFonts.size(); ++i)
	{
		QFont font;
		font.fromString(itemFonts.at(i));
		itemTable->item(i, 0)->setFont(font);

		QFontMetrics metrics(font);
		itemTable->setRowHeight(i, metrics.height() + itemMargin);
	}
}

void AppearanceWidget::setItemColors(const QStringList &colorNames)
{
	itemColors = colorNames;
	for (int i = 0; i < itemColors.size(); ++i)
	{
		QColor color(itemColors.at(i));
		itemTable->item(i, 0)->setForeground(color);
	}
}

QStringList AppearanceWidget::getItemFonts()
{
	return itemFonts;
}

QStringList AppearanceWidget::getItemColors()
{
	return itemColors;
}

void AppearanceWidget::setCustomizationType(bool custom)
{
	if (custom)
		customAppearanceCheck->setChecked(true);
	else
		standardAppearanceCheck->setChecked(true);
	toggleCustom();
}

bool AppearanceWidget::getCustomizationType()
{
	return !standardAppearanceCheck->isChecked();
}

void AppearanceWidget::toggleCustom()
{
	if (customAppearanceCheck->isChecked())
	{
		appearanceWidget->setEnabled(true);
		for (int i = 0; i < itemColors.size(); ++i)
		{
			QColor color(itemColors.at(i));
			itemTable->item(i, 0)->setForeground(color);
		}
	}
	else
	{
		appearanceWidget->setEnabled(false);
		for (int i = 0; i < itemColors.size(); ++i)
		{
			QColor color(QApplication::style()->standardPalette().color(QPalette::Disabled, QPalette::Text));
			itemTable->item(i, 0)->setForeground(color);
		}
	}
}

void AppearanceWidget::setTypeNames(const QStringList &typeNames)
{
	this->typeNames = typeNames;
}

void AppearanceWidget::setDefaultTextCharFormats(const QMap<QString, QTextCharFormat> &defaultFormatList)
{
	for (int i = 0; i < defaultFormatList.size(); ++i)
	{
		itemColors << "";
		itemFonts << "";
	}

	QMap<QString, QTextCharFormat>::const_iterator it = defaultFormatList.constBegin();
	int num;
	while (it != defaultFormatList.constEnd())
	{
		num = typeNames.indexOf(it.key());
		if (num < 0) continue;
		itemColors[num] = it.value().foreground().color().name();
		itemFonts[num] = it.value().font().toString();
		++it;
	}
}

void AppearanceWidget::readSettings(const QString &settingsGroup)
{
	int num;
	QSettings settings;
	settings.beginGroup(settingsGroup);
	custom = settings.value("Customize", true).toBool();
	int numOfRules = settings.value("Number", 0).toInt();
	for (int i = 0; i < numOfRules; ++i)
	{
		QString name = settings.value("Item" + QString::number(i) + "/Name").toString();
		QString colorName = settings.value("Item" + QString::number(i) + "/Color").toString();
		QString fontName = settings.value("Item" + QString::number(i) + "/Font").toString();
		num = typeNames.indexOf(name);
		if (num >= 0)
		{
			itemColors[num] = colorName;
			itemFonts[num] = fontName;
		}
	}
	settings.endGroup();
}

void AppearanceWidget::setSettings()
{
	setCustomizationType(custom);
	setItemFonts(itemFonts);
	setItemColors(itemColors);
}

void AppearanceWidget::writeSettings(const QString &settingsGroup)
{
	custom = getCustomizationType();

	QSettings settings;
	settings.beginGroup(settingsGroup);
	settings.setValue("Customize", custom);
	if (custom)
	{
		for (int i = 0; i < typeNames.size(); ++i)
		{
			settings.setValue("Item" + QString::number(i) + "/Name", typeNames.at(i));
			settings.setValue("Item" + QString::number(i) + "/Color", itemColors.at(i));
			settings.setValue("Item" + QString::number(i) + "/Font", itemFonts.at(i));
		}
		settings.setValue("Number", typeNames.size());
	}
	settings.endGroup();
}

void AppearanceWidget::showFontDialog()
{
	if (itemHighlighted < 0) return;

	bool ok;
	QFont currentFont;
	currentFont.fromString(itemFonts.at(itemHighlighted));
	QFont newFont = QFontDialog::getFont(&ok, currentFont, this);
	if (ok)
	{
		QTableWidgetItem *item = itemTable->item(itemHighlighted, 0);
		itemFonts.replace(itemHighlighted, newFont.toString());
		item->setFont(newFont);
		setItemToolTip(item, newFont);
	}
}

void AppearanceWidget::showColorDialog()
{
	if (itemHighlighted < 0) return;

	QColor currentColor(itemColors.at(itemHighlighted));
	QColor newColor = QColorDialog::getColor(currentColor, this);
	if (newColor.isValid())
	{
		itemColors.replace(itemHighlighted, newColor.name());
		itemTable->item(itemHighlighted, 0)->setForeground(newColor);
		setHighlightedForeground(itemColors.at(itemHighlighted));
	}
}

void AppearanceWidget::setHighlightedForeground(const QString &colorName)
{
	QPalette palette = itemTable->palette();
	QColor color(colorName);
	palette.setColor(QPalette::Normal, QPalette::HighlightedText, color);
	palette.setColor(QPalette::Inactive, QPalette::HighlightedText, color);
	itemTable->setPalette(palette);
}
