/***************************************************************************
 *   Copyright (C) 2007 by Glad Deschrijver                                *
 *   Glad.Deschrijver@UGent.be                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QApplication>
#include <QDialogButtonBox>
#include <QDir>
#include <QFileDialog>
#include <QFontDialog>
#include <QKeyEvent>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QSettings>
#include <QSpinBox>
#include <QTabWidget>
#include <QVBoxLayout>
#include <QWhatsThis>

#include "appearancewidget.h"
#include "configdialog.h"

ConfigDialog::ConfigDialog(QWidget *parent) : QDialog(parent)
{
	setWindowTitle(tr("Configure TikZ editor"));

	centerWidget = new QTabWidget;
	centerWidget->addTab(generalPage(), tr("&General"));
	centerWidget->addTab(appearancePage(this), tr("&Highlighting"));

	buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok
	    | QDialogButtonBox::Cancel);
	connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
	connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));

	QVBoxLayout *mainLayout = new QVBoxLayout;
	mainLayout->addWidget(centerWidget);
	mainLayout->addWidget(buttonBox);
	setLayout(mainLayout);
}

void ConfigDialog::setTranslatedTypeNames(const QStringList &typeNames)
{
	for (int i = 0; i < typeNames.size(); ++i)
		appearance->addItem(typeNames.at(i));
}

void ConfigDialog::setTypeNames(const QStringList &typeNames)
{
	appearance->setTypeNames(typeNames);
}

void ConfigDialog::setDefaultTextCharFormats(const QMap<QString, QTextCharFormat> &defaultFormatList)
{
	appearance->setDefaultTextCharFormats(defaultFormatList);
}

void ConfigDialog::readSettings()
{
	appearance->readSettings("Highlighting");

	QSettings settings;
	numRecentFiles = settings.value("RecentFilesNumber", 5).toInt();
	latexCommand = settings.value("LatexCommand", "pdflatex").toString();
	pdftopsCommand = settings.value("PdftopsCommand", "pdftops").toString();

	settings.beginGroup("Editor");
	QString textFontString = settings.value("Font", qApp->font().toString()).toString();
	textFont.fromString(textFontString);
	settings.endGroup();

	setSettings();
}

void ConfigDialog::setSettings()
{
	textFontEdit->setText(textFont.family() + " " + QString::number(textFont.pointSize()));
	textFontEdit->setFont(textFont);
	historyLengthSpinBox->setValue(numRecentFiles);
	latexEdit->setText(latexCommand);
	pdftopsEdit->setText(pdftopsCommand);

	appearance->setSettings();
}

void ConfigDialog::writeSettings()
{
	numRecentFiles = historyLengthSpinBox->value();
	latexCommand = latexEdit->text();
	pdftopsCommand = pdftopsEdit->text();

	QSettings settings;
	settings.setValue("RecentFilesNumber", numRecentFiles);
	settings.setValue("LatexCommand", latexCommand);
	settings.setValue("PdftopsCommand", pdftopsCommand);

	settings.beginGroup("Editor");
	settings.setValue("Font", textFont.toString());
	settings.endGroup();

	appearance->writeSettings("Highlighting");
}

QWidget *ConfigDialog::generalPage()
{
	QWidget *page = new QWidget;

	QString textFontWhatsThis = "<p>" + tr("Select the font of the main text.") + "</p>";
	QLabel *textFontLabel = new QLabel(tr("Text &font:"));
	textFontEdit = new QLineEdit;
	textFontEdit->setReadOnly(true);
	textFontEdit->setWhatsThis(textFontWhatsThis);
	textFontLabel->setWhatsThis(textFontWhatsThis);
	QPushButton *textFontButton = new QPushButton(tr("Select"));
	textFontButton->setToolTip(tr("Select font"));
	textFontButton->setWhatsThis("<p>" + tr("Select the font of the main text.") + "</p>");
	textFontLabel->setBuddy(textFontButton);
	connect(textFontButton, SIGNAL(clicked()), this, SLOT(selectFont()));
	QWidget *textFontWidget = new QWidget;
	QHBoxLayout *textFontLayout = new QHBoxLayout;
	textFontLayout->addWidget(textFontLabel);
	textFontLayout->addWidget(textFontEdit);
	textFontLayout->addWidget(textFontButton);
	textFontLayout->setMargin(0);
	textFontWidget->setLayout(textFontLayout);

	QLabel *historyLengthLabel = new QLabel(tr("&Number of entries in the \"Open Recent\" menu:"));
	historyLengthSpinBox = new QSpinBox();
	QString historyLengthWhatsThis = "<p>" + tr("Select the maximum number "
	    "of entries that appear in the \"Open Recent\" menu.  When "
	    "the number of recently opened files exceeds this limit, "
	    "the oldest entries will be lost.") + "</p>";
	historyLengthLabel->setWhatsThis(historyLengthWhatsThis);
	historyLengthLabel->setBuddy(historyLengthSpinBox);
	historyLengthSpinBox->setWhatsThis(historyLengthWhatsThis);
	QWidget *historyLengthBox = new QWidget;
	QHBoxLayout *historyLengthLayout = new QHBoxLayout;
	historyLengthLayout->addWidget(historyLengthLabel);
	historyLengthLayout->addWidget(historyLengthSpinBox);
	historyLengthLayout->setMargin(0);
	historyLengthBox->setLayout(historyLengthLayout);

	QString latexWhatsThis = "<p>" + tr("Enter the path to the LaTeX executable here.") + "</p>";
	QLabel *latexLabel = new QLabel(tr("&LaTeX command:"));
	latexEdit = new QLineEdit;
	latexEdit->setWhatsThis(latexWhatsThis);
	latexLabel->setWhatsThis(latexWhatsThis);
	latexLabel->setBuddy(latexEdit);
	QPushButton *latexButton = new QPushButton;
	latexButton->setIcon(QIcon(":/images/document-open.png"));
	latexButton->setToolTip(tr("Browse command"));
	latexButton->setWhatsThis("<p>" + tr("Browse to the LaTeX executable.") + "</p>");
	latexButton->setObjectName("latexButton");
	connect(latexButton, SIGNAL(clicked()), this, SLOT(browseCommand()));

	QString pdftopsWhatsThis = "<p>" + tr("Enter the path to the pdftops executable here.") + "</p>";
	QLabel *pdftopsLabel = new QLabel(tr("&Pdftops command:"));
	pdftopsEdit = new QLineEdit;
	pdftopsEdit->setWhatsThis(pdftopsWhatsThis);
	pdftopsLabel->setWhatsThis(pdftopsWhatsThis);
	pdftopsLabel->setBuddy(pdftopsEdit);
	QPushButton *pdftopsButton = new QPushButton;
	pdftopsButton->setIcon(QIcon(":/images/document-open.png"));
	pdftopsButton->setToolTip(tr("Browse command"));
	pdftopsButton->setWhatsThis("<p>" + tr("Browse to the pdftops executable.") + "</p>");
	pdftopsButton->setObjectName("pdftopsButton");
	connect(pdftopsButton, SIGNAL(clicked()), this, SLOT(browseCommand()));

	QGridLayout *commandsLayout = new QGridLayout;
	commandsLayout->addWidget(latexLabel, 0, 0);
	commandsLayout->addWidget(latexEdit, 0, 1);
	commandsLayout->addWidget(latexButton, 0, 2);
	commandsLayout->addWidget(pdftopsLabel, 1, 0);
	commandsLayout->addWidget(pdftopsEdit, 1, 1);
	commandsLayout->addWidget(pdftopsButton, 1, 2);
	commandsLayout->setMargin(0);
	QWidget *commandsWidget = new QWidget;
	commandsWidget->setLayout(commandsLayout);

	// add vertical spacer
	QWidget *vspacer = new QWidget;
	vspacer->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding);

	QVBoxLayout *layout = new QVBoxLayout;
	layout->addWidget(textFontWidget);
	layout->addWidget(historyLengthBox);
	layout->addWidget(commandsWidget);
	layout->addWidget(vspacer);
	page->setLayout(layout);

	return page;
}

QWidget *ConfigDialog::appearancePage(QWidget *parent)
{
	appearance = new AppearanceWidget(parent);

	QWidget *page = new QWidget;
	QVBoxLayout *layout = new QVBoxLayout;
	layout->addWidget(appearance);
	page->setLayout(layout);

	return page;
}

void ConfigDialog::selectFont()
{
	bool ok;
	QFont newFont = QFontDialog::getFont(&ok, textFont, this);
	if (&ok)
	{
		textFont = newFont;
		textFontEdit->setText(textFont.family() + " " + QString::number(textFont.pointSize()));
		textFontEdit->setFont(textFont);
	}
}

void ConfigDialog::browseCommand(QLineEdit *lineEdit)
{
	QString location = QFileDialog::getOpenFileName(this,
	    tr("Browse program"), QDir::rootPath(),
	    QString("%1 (*)").arg(tr("Program")), 0,
	    QFileDialog::DontResolveSymlinks);
	if (!location.isEmpty())
	{
		location.replace("\\", "/");
		lineEdit->setText(location);
	}
}

void ConfigDialog::browseCommand()
{
	QPushButton *button = qobject_cast<QPushButton*>(sender());
	if (button->objectName() == "latexButton")
		browseCommand(latexEdit);
	else if (button->objectName() == "pdftopsButton")
		browseCommand(pdftopsEdit);
}

void ConfigDialog::accept()
{
	writeSettings();
	emit settingsChanged();
	QDialog::accept();
}

void ConfigDialog::keyPressEvent(QKeyEvent *event)
{
	if (event->key() == Qt::Key_F1 && event->modifiers() == Qt::ShiftModifier)
	{
		QWhatsThis::enterWhatsThisMode();
	}
	QDialog::keyPressEvent(event);
}
