/***************************************************************************
 *   Copyright (C) 2007 by Florian Hackenberger                            *
 *   Copyright (C) 2007 by Glad Deschrijver                                *
 *   florian@hackenberger.at                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QDebug>
#include <QAction>
#include <QApplication>
#include <QClipboard>
#include <QCloseEvent>
#include <QDockWidget>
#include <QFileDialog>
#include <QLabel>
#include <QMenuBar>
#include <QMessageBox>
#include <QSettings>
#include <QSplitter>
#include <QStatusBar>
#include <QTextCursor>
#include <QTextEdit>
#include <QTextStream>
#include <QToolBar>
#include <QWhatsThis>

#include "aboutdialog.h"
#include "configdialog.h"
#include "ktikz.h"
#include "latexloghighlighter.h"
#include "latexlogtextedit.h"
#include "tikzhighlighter.h"
#include "tikzpreview.h"

#define UNSTABLE_POPPLER_QT4
#include <poppler-qt4.h>

ktikz::ktikz()
{
	aboutDialog = 0;
	configDialog = 0;
	tikzPdfDoc = 0;

	setWindowIcon(QIcon(":/images/ktikz-22.png"));

	textEdit = new QTextEdit;
	textEdit->setWhatsThis("<p>" + tr("Enter your TikZ code here.  "
	    "The code should begin with \\begin{tikzpicture} and end with "
	    "\\end{tikzpicture}.") + "</p>");
	commandInserter = new TikzCommandInserter(textEdit, this);
	tikzHighlighter = new TikzHighlighter(commandInserter, textEdit->document());
	tikzHighlighter->rehighlight(); // avoid that textEdit emits the signal contentsChanged() when it is still empty
	tikzView = new TikzPreview(this);
	tikzController = new TikzPngPreviewer(textEdit);

	logDock = new QDockWidget(this);
	logDock->setObjectName("LogDock");
	logDock->setAllowedAreas(Qt::AllDockWidgetAreas);
	logDock->setFeatures(QDockWidget::DockWidgetClosable | QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);
	logDock->setWindowTitle(tr("Messages"));
	addDockWidget(Qt::BottomDockWidgetArea, logDock);
	logWindow = new LogTextEdit;
	logWindow->setWhatsThis("<p>" + tr("The messages produced by "
	    "LaTeX are shown here.  If your TikZ code contains errors, "
	    "then a red border will appear and the errors will be "
	    "highlighted.") + "</p>");
	logWindow->setReadOnly(true);
	logHighlighter = new LatexLogHighlighter(logWindow->document());
	logDock->setWidget(logWindow);

	QList<int> splitterSizes;
	splitterSizes << 350 << 250;
	splitter = new QSplitter(Qt::Horizontal, this);
	splitter->addWidget(textEdit);
	splitter->addWidget(tikzView);
	splitter->setSizes(splitterSizes);
	setCentralWidget(splitter);

	createActions();
	createToolBars(); // run first in order to be able to add file/editToolBar->toggleViewAction() to the menu
	createMenus();
	createStatusBar();

	readSettings(); // must be run after defining tikzController and tikzHighlighter, and after creating the toolbars

	connect(textEdit->document(), SIGNAL(modificationChanged(bool)),
	        this, SLOT(setDocumentModified(bool)));
	connect(textEdit->document(), SIGNAL(contentsChanged()),
	        tikzController, SLOT(regeneratePreview()));
	connect(textEdit, SIGNAL(cursorPositionChanged()),
		this, SLOT(showCursorPosition()));
	connect(tikzController, SIGNAL(pixmapUpdated()),
	        this, SLOT(pixmapUpdated()));
	connect(tikzController, SIGNAL(shortLogUpdated()),
	        this, SLOT(shortLogUpdated()));

	setCurrentFile("");
	setDocumentModified(false);
	textEdit->setFont(editFont);
	textEdit->setFocus();

	QDir dir(QDir::tempPath() + "/ktikz");
	if (!dir.exists())
		QDir::temp().mkdir("ktikz");
}

ktikz::~ktikz()
{
	writeSettings();

	//TODO delete stuff from constructor
	delete tikzController;
	delete logHighlighter;
	delete tikzHighlighter;

	QDir dir(QDir::tempPath() + "/ktikz");
	if (dir.exists())
		QDir::temp().rmdir("ktikz");
}

void ktikz::closeEvent(QCloseEvent *event)
{
	if (maybeSave()) {
		event->accept();
	} else {
		event->ignore();
	}
}

void ktikz::newFile()
{
	if (maybeSave()) {
		textEdit->clear();
		setCurrentFile("");
	}
}

void ktikz::open()
{
	if (maybeSave()) {
		QString lastDir;
		if (!lastDocument.isEmpty())
		{
			QFileInfo lastFileInfo(lastDocument);
			lastDir = lastFileInfo.absolutePath();
		}
		QString fileName = QFileDialog::getOpenFileName(this, tr("Open PGF source file"), lastDir);
		if (!fileName.isEmpty())
			loadFile(fileName);
	}
}

bool ktikz::save()
{
	if (curFile.isEmpty()) {
		return saveAs();
	} else {
		return saveFile(curFile);
	}
}

bool ktikz::saveAs()
{
	QString lastDir;
	if (!lastDocument.isEmpty())
	{
		QFileInfo lastFileInfo(lastDocument);
		lastDir = lastFileInfo.absolutePath();
	}
	QString fileName = QFileDialog::getSaveFileName(this, tr("Save PGF source file"), lastDir);
	if (fileName.isEmpty())
		return false;
	
	return saveFile(fileName);
}

bool ktikz::exportImage()
{
	QAction *Act = qobject_cast<QAction*>(sender());

	QPixmap *tikzImage = tikzView->getPixmap();
	if (tikzImage->isNull()) return false;

	QString lastDir;
	if (!lastDocument.isEmpty())
	{
		QFileInfo lastFileInfo(lastDocument);
		lastDir = lastFileInfo.absolutePath();
	}
	QString filter = QString("%1 (*." + Act->data().toString() + ");;%2 (*.*)").arg(Act->text()).arg(tr("All files"));
	QString fileName = QFileDialog::getSaveFileName(this, tr("Export image"), lastDir, filter);
	if (!fileName.isEmpty())
	{
		QString type = Act->data().toString();
		if (type == "png")
		{
			if (!fileName.endsWith(".png"))
				fileName = fileName + ".png";
			return tikzImage->save(fileName, "PNG");
		}
		else
			return tikzController->exportImage(fileName, type);
	}
	return false;
}

void ktikz::about()
{
	if (!aboutDialog)
		aboutDialog = new AboutDialog(this);
	aboutDialog->exec();
}

void ktikz::setDocumentModified(bool isModified)
{
	setWindowModified(isModified);
}

void ktikz::pixmapUpdated()
{
	tikzPdfDoc = tikzController->getPdfDocument();
	tikzView->pixmapUpdated(tikzPdfDoc);
}

void ktikz::shortLogUpdated()
{
	logWindow->setPlainText(tikzController->getShortLogText());
	setLogPalette();
}

void ktikz::logUpdated()
{
	logWindow->setPlainText(tikzController->getLogText());
	setLogPalette();
}

void ktikz::setLogPalette()
{
	logWindow->moveCursor(QTextCursor::End);
	if(tikzController->hasRunFailed()) {
		QPalette failedPalette(QApplication::palette());
		failedPalette.setColor(QPalette::Background, QColor(255, 102, 102));
		logWindow->setAutoFillBackground(true);
		logWindow->setPalette(failedPalette);
	}else {
		logWindow->setPalette(QApplication::palette());
	}
}

void ktikz::createActions()
{
	QString newWhatsThis = tr("Create a new document");
	newAct = new QAction(QIcon(":/images/document-new.png"), tr("&New"), this);
	newAct->setShortcut(QKeySequence::New);
	newAct->setStatusTip(newWhatsThis);
	newAct->setWhatsThis("<p>" + newWhatsThis + "</p>");
	connect(newAct, SIGNAL(triggered()), this, SLOT(newFile()));

	QString openWhatsThis = tr("Open an existing file");
	openAct = new QAction(QIcon(":/images/document-open.png"), tr("&Open..."), this);
	openAct->setShortcut(QKeySequence::Open);
	openAct->setStatusTip(openWhatsThis);
	openAct->setWhatsThis("<p>" + openWhatsThis + "</p>");
	connect(openAct, SIGNAL(triggered()), this, SLOT(open()));

	QString saveWhatsThis = tr("Save the current document to disk");
	saveAct = new QAction(QIcon(":/images/document-save.png"), tr("&Save"), this);
	saveAct->setShortcut(QKeySequence::Save);
	saveAct->setStatusTip(saveWhatsThis);
	saveAct->setWhatsThis("<p>" + saveWhatsThis + "</p>");
	connect(saveAct, SIGNAL(triggered()), this, SLOT(save()));

	QString saveAsWhatsThis = tr("Save the document under a new name");
	saveAsAct = new QAction(QIcon(":/images/document-save-as.png"), tr("Save &As..."), this);
	saveAsAct->setStatusTip(saveAsWhatsThis);
	saveAsAct->setWhatsThis("<p>" + saveAsWhatsThis + "</p>");
	connect(saveAsAct, SIGNAL(triggered()), this, SLOT(saveAs()));

	QString exitWhatsThis = tr("Exit the application");
	exitAct = new QAction(QIcon(":/images/application-exit.png"), tr("E&xit"), this);
	exitAct->setShortcut(tr("Ctrl+Q", "Quit"));
	exitAct->setStatusTip(exitWhatsThis);
	exitAct->setWhatsThis("<p>" + exitWhatsThis + "</p>");
	connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

	exportEpsAct = new QAction(QIcon(":/images/image-x-eps.png"), tr("Encapsulated PostScript (EPS)"), this);
	exportEpsAct->setData("eps");
	exportEpsAct->setStatusTip(tr("Export to PostScript"));
	connect(exportEpsAct, SIGNAL(triggered()), this, SLOT(exportImage()));

	exportPdfAct = new QAction(QIcon(":/images/application-pdf.png"), tr("Portable Document Format (PDF)"), this);
	exportPdfAct->setData("pdf");
	exportPdfAct->setStatusTip(tr("Export to PDF"));
	connect(exportPdfAct, SIGNAL(triggered()), this, SLOT(exportImage()));

	exportPngAct = new QAction(QIcon(":/images/image-png.png"), tr("Portable Network Graphics (PNG)"), this);
	exportPngAct->setData("png");
	exportPngAct->setStatusTip(tr("Export to PNG"));
	connect(exportPngAct, SIGNAL(triggered()), this, SLOT(exportImage()));

	QString undoWhatsThis = tr("Undo the previous action");
	undoAct = new QAction(QIcon(":/images/edit-undo.png"), tr("&Undo"), this);
	undoAct->setShortcut(QKeySequence::Undo);
	undoAct->setStatusTip(undoWhatsThis);
	undoAct->setWhatsThis("<p>" + undoWhatsThis + "</p>");
	connect(undoAct, SIGNAL(triggered()), textEdit, SLOT(undo()));

	QString redoWhatsThis = tr("Redo the previous undone action");
	redoAct = new QAction(QIcon(":/images/edit-redo.png"), tr("Re&do"), this);
	redoAct->setShortcut(QKeySequence::Redo);
	redoAct->setStatusTip(redoWhatsThis);
	redoAct->setWhatsThis("<p>" + redoWhatsThis + "</p>");
	connect(redoAct, SIGNAL(triggered()), textEdit, SLOT(redo()));

	QString cutWhatsThis = tr("Cut the current selection's contents to the clipboard");
	cutAct = new QAction(QIcon(":/images/edit-cut.png"), tr("Cu&t"), this);
	cutAct->setShortcut(QKeySequence::Cut);
	cutAct->setStatusTip(cutWhatsThis);
	cutAct->setWhatsThis("<p>" + cutWhatsThis + "</p>");
	connect(cutAct, SIGNAL(triggered()), textEdit, SLOT(cut()));

	QString copyWhatsThis = tr("Copy the current selection's contents to the clipboard");
	copyAct = new QAction(QIcon(":/images/edit-copy.png"), tr("&Copy"), this);
	copyAct->setShortcut(QKeySequence::Copy);
	copyAct->setStatusTip(copyWhatsThis);
	copyAct->setWhatsThis("<p>" + copyWhatsThis + "</p>");
	connect(copyAct, SIGNAL(triggered()), textEdit, SLOT(copy()));

	QString pasteWhatsThis = tr("Paste the clipboard's contents into the current selection");
	pasteAct = new QAction(QIcon(":/images/edit-paste.png"), tr("&Paste"), this);
	pasteAct->setShortcut(QKeySequence::Paste);
	pasteAct->setStatusTip(pasteWhatsThis);
	pasteAct->setWhatsThis("<p>" + pasteWhatsThis + "</p>");
	connect(pasteAct, SIGNAL(triggered()), textEdit, SLOT(paste()));

	selectAllAct = new QAction(tr("Select &All"), this);
	selectAllAct->setShortcut(QKeySequence::SelectAll);
	selectAllAct->setStatusTip(tr("Select all the content"));
	connect(selectAllAct, SIGNAL(triggered()), textEdit, SLOT(selectAll()));

	commentAct = new QAction(tr("C&omment"), this);
	commentAct->setShortcut(tr("Ctrl+D", "Comment"));
	commentAct->setStatusTip(tr("Comment the current line or selection"));
	connect(commentAct, SIGNAL(triggered()), this, SLOT(editComment()));

	unCommentAct = new QAction(tr("Unco&mment"), this);
	unCommentAct->setShortcut(tr("Ctrl+Shift+D", "Uncomment"));
	unCommentAct->setStatusTip(tr("Uncomment the current line or selection"));
	connect(unCommentAct, SIGNAL(triggered()), this, SLOT(editUncomment()));

	procStopAct = new QAction(QIcon(":/images/process-stop.png"), tr("&Stop Process"), this);
	procStopAct->setShortcut(tr("Escape", "Stop process"));
	procStopAct->setStatusTip(tr("Abort current process"));
	procStopAct->setWhatsThis("<p>" + tr("Abort the execution of the currently running process.") + "</p>");
	connect(procStopAct, SIGNAL(triggered()), tikzController, SLOT(abortProcess()));

	viewLogAct = new QAction(QIcon(":/images/compfile.png"), tr("View &Log"), this);
	viewLogAct->setStatusTip(tr("View log messages produced by the last executed process"));
	viewLogAct->setWhatsThis("<p>" + tr("Show the log messages produced by the last executed process in the Messages box.") + "</p>");
	connect(viewLogAct, SIGNAL(triggered()), this, SLOT(logUpdated()));

	configureAct = new QAction(QIcon(":/images/configure.png"), tr("&Configure"), this);
	configureAct->setStatusTip(tr("Configure the settings of this application"));
	connect(configureAct, SIGNAL(triggered()), this, SLOT(configure()));

	aboutAct = new QAction(QIcon(":/images/ktikz-22.png"), tr("&About"), this);
	aboutAct->setStatusTip(tr("Show the application's About box"));
	connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

	aboutQtAct = new QAction(tr("About &Qt"), this);
	aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
	connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

	whatsThisAct = QWhatsThis::createAction(this);
	whatsThisAct->setIcon(QIcon(":/images/help-whatsthis.png"));
	whatsThisAct->setStatusTip(tr("Show simple description of any widget"));

	undoAct->setEnabled(false);
	redoAct->setEnabled(false);
	cutAct->setEnabled(false);
	copyAct->setEnabled(false);
	pasteAct->setEnabled(textEdit->canPaste());
	procStopAct->setEnabled(false);
	connect(textEdit, SIGNAL(undoAvailable(bool)),
	        undoAct, SLOT(setEnabled(bool)));
	connect(textEdit, SIGNAL(redoAvailable(bool)),
	        redoAct, SLOT(setEnabled(bool)));
	connect(textEdit, SIGNAL(copyAvailable(bool)),
	        cutAct, SLOT(setEnabled(bool)));
	connect(textEdit, SIGNAL(copyAvailable(bool)),
	        copyAct, SLOT(setEnabled(bool)));
	connect(QApplication::clipboard(), SIGNAL(dataChanged()),
	        this, SLOT(setPasteEnabled()));
	connect(tikzController, SIGNAL(processRunning(bool)),
	        procStopAct, SLOT(setEnabled(bool)));
}

void ktikz::createMenus()
{
	recentMenu = new QMenu(tr("Open &Recent"));
	recentMenu->setIcon(QIcon(":/images/document-open-recent.png"));
	recentMenu->menuAction()->setStatusTip(tr("Open a recently opened file"));

	exportMenu = new QMenu(tr("&Export"));
	exportMenu->setIcon(QIcon(":/images/file-export.png"));
	exportMenu->menuAction()->setStatusTip(tr("Export image to various formats"));
	exportMenu->addAction(exportEpsAct);
	exportMenu->addAction(exportPdfAct);
	exportMenu->addAction(exportPngAct);

	fileMenu = menuBar()->addMenu(tr("&File"));
	fileMenu->addAction(newAct);
	fileMenu->addAction(openAct);
	fileMenu->addMenu(recentMenu);
	fileMenu->addSeparator();
	fileMenu->addAction(saveAct);
	fileMenu->addAction(saveAsAct);
	fileMenu->addMenu(exportMenu);
	fileMenu->addSeparator();
	fileMenu->addAction(exitAct);

	editMenu = menuBar()->addMenu(tr("&Edit"));
	editMenu->addAction(undoAct);
	editMenu->addAction(redoAct);
	editMenu->addSeparator();
	editMenu->addAction(cutAct);
	editMenu->addAction(copyAct);
	editMenu->addAction(pasteAct);
	editMenu->addAction(selectAllAct);
	editMenu->addSeparator();
	editMenu->addAction(commentAct);
	editMenu->addAction(unCommentAct);

	viewMenu = menuBar()->addMenu(tr("&View"));
	viewMenu->addActions(tikzView->getActions());
	viewMenu->addSeparator();
	viewMenu->addAction(procStopAct);
	viewMenu->addAction(viewLogAct);

	menuBar()->addMenu(commandInserter->getMenu());

	settingsMenu = menuBar()->addMenu(tr("&Settings"));
	settingsMenu->addAction(fileToolBar->toggleViewAction());
	settingsMenu->addAction(editToolBar->toggleViewAction());
	settingsMenu->addAction(viewToolBar->toggleViewAction());
	settingsMenu->addAction(runToolBar->toggleViewAction());
	settingsMenu->addSeparator();
	settingsMenu->addAction(logDock->toggleViewAction());
	settingsMenu->addSeparator();
	settingsMenu->addAction(configureAct);
	connect(fileToolBar->toggleViewAction(), SIGNAL(toggled(bool)), this, SLOT(setToolBarStatusTip(bool)));
	connect(editToolBar->toggleViewAction(), SIGNAL(toggled(bool)), this, SLOT(setToolBarStatusTip(bool)));
	connect(viewToolBar->toggleViewAction(), SIGNAL(toggled(bool)), this, SLOT(setToolBarStatusTip(bool)));
	connect(runToolBar->toggleViewAction(), SIGNAL(toggled(bool)), this, SLOT(setToolBarStatusTip(bool)));
	connect(logDock->toggleViewAction(), SIGNAL(toggled(bool)), this, SLOT(setDockWidgetStatusTip(bool)));

	menuBar()->addSeparator();

	helpMenu = menuBar()->addMenu(tr("&Help"));
	helpMenu->addAction(whatsThisAct);
	helpMenu->addSeparator();
	helpMenu->addAction(aboutAct);
	helpMenu->addAction(aboutQtAct);
}

void ktikz::createToolBars()
{
	fileToolBar = addToolBar(tr("File"));
	fileToolBar->setObjectName("FileToolBar");
	fileToolBar->addAction(newAct);
	fileToolBar->addAction(openAct);
	fileToolBar->addAction(saveAct);
	
	editToolBar = addToolBar(tr("Edit"));
	editToolBar->setObjectName("EditToolBar");
	editToolBar->addAction(undoAct);
	editToolBar->addAction(redoAct);
	editToolBar->addAction(cutAct);
	editToolBar->addAction(copyAct);
	editToolBar->addAction(pasteAct);

	viewToolBar = tikzView->getViewToolBar();
	addToolBar(viewToolBar);

	runToolBar = addToolBar(tr("Run"));
	runToolBar->setObjectName("RunToolBar");
	runToolBar->addAction(procStopAct);
	runToolBar->addAction(viewLogAct);
}

void ktikz::createStatusBar()
{
	QStatusBar *status = statusBar();

	positionLabel = new QLabel(status);
	positionLabel->setMinimumWidth(150);
	status->addPermanentWidget(positionLabel, 0);

	status->showMessage(tr("Ready"));
}

void ktikz::setDockWidgetStatusTip(bool enabled)
{
	QAction *Act = qobject_cast<QAction*>(sender());
	if (enabled)
		Act->setStatusTip(tr("Hide sidebar \"%1\"").arg(Act->text()));
	else
		Act->setStatusTip(tr("Show sidebar \"%1\"").arg(Act->text()));
}

void ktikz::setToolBarStatusTip(bool enabled)
{
	QAction *Act = qobject_cast<QAction*>(sender());
	if (enabled)
		Act->setStatusTip(tr("Hide toolbar \"%1\"").arg(Act->text()));
	else
		Act->setStatusTip(tr("Show toolbar \"%1\"").arg(Act->text()));
}

void ktikz::setPasteEnabled()
{
	pasteAct->setEnabled(textEdit->canPaste());
}

void ktikz::configure()
{
	if (!configDialog)
	{
		configDialog = new ConfigDialog(this);
		configDialog->setTranslatedTypeNames(tikzHighlighter->getTranslatedTypeNames());
		configDialog->setTypeNames(tikzHighlighter->getTypeNames());
		configDialog->setDefaultTextCharFormats(tikzHighlighter->getDefaultTextCharFormats());
		connect(configDialog, SIGNAL(settingsChanged()), this, SLOT(applySettings()));
	}
	configDialog->readSettings();
	configDialog->exec();
}

void ktikz::applySettings()
{
	QSettings settings;

	numOfRecentFiles = settings.value("RecentFilesNumber", 5).toInt();
	QString latexCommand = settings.value("LatexCommand", "pdflatex").toString();
	QString pdftopsCommand = settings.value("PdftopsCommand", "pdftops").toString();
	tikzController->setLatexCommand(latexCommand);
	tikzController->setPdftopsCommand(pdftopsCommand);

	settings.beginGroup("Editor");
	QString fontText = settings.value("Font", qApp->font().toString()).toString();
	editFont.fromString(fontText);
	textEdit->setFont(editFont);
	settings.endGroup();

	settings.beginGroup("Highlighting");
	customHighlighting = settings.value("Customize", true).toBool();
	QMap<QString, QTextCharFormat> formatList = tikzHighlighter->getDefaultTextCharFormats();
	if (customHighlighting)
	{
		int numOfRules = settings.value("Number", 0).toInt();
		for (int i = 0; i < numOfRules; ++i)
		{
			QString name = settings.value("Item" + QString::number(i) + "/Name").toString();
			QString colorName = settings.value("Item" + QString::number(i) + "/Color").toString();
			QString fontName = settings.value("Item" + QString::number(i) + "/Font").toString();
			QFont font;
			font.fromString(fontName);
			QTextCharFormat format;
			format.setForeground(QBrush(QColor(colorName)));
			format.setFont(font);
			formatList[name] = format;
		}
	}
	settings.endGroup();

	tikzHighlighter->setTextCharFormats(formatList);
	tikzHighlighter->rehighlight();

	createRecentFilesList();
}

void ktikz::readSettings()
{
	QSettings settings;
	settings.beginGroup("MainWindow");
	QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
	QSize size = settings.value("size", QSize(600, 450)).toSize();
	resize(size);
	move(pos);
	restoreState(settings.value("MainWindowState").toByteArray());
	splitter->restoreState(settings.value("SplitterState").toByteArray());
	settings.endGroup();

	recentFilesList = settings.value("RecentFiles").toStringList();
	lastDocument = settings.value("LastDocument").toString();

	applySettings();
}

void ktikz::writeSettings()
{
	QSettings settings;
	settings.beginGroup("MainWindow");
	settings.setValue("pos", pos());
	settings.setValue("size", size());
	settings.setValue("MainWindowState", saveState());
	settings.setValue("SplitterState", splitter->saveState());
	settings.endGroup();

	if (recentFilesList.size() > 0)
		settings.setValue("RecentFiles", recentFilesList);
	settings.setValue("LastDocument", lastDocument);
}

bool ktikz::maybeSave()
{
	if (textEdit->document()->isModified()) {
		int ret = QMessageBox::warning(this, tr("TikZ editor"),
		                               tr("The document has been modified.\n"
		                                  "Do you want to save your changes?"),
		                               QMessageBox::Yes | QMessageBox::Default,
		                               QMessageBox::No,
		                               QMessageBox::Cancel | QMessageBox::Escape);
		if (ret == QMessageBox::Yes)
			return save();
		else if (ret == QMessageBox::Cancel)
			return false;
	}
	return true;
}

void ktikz::loadFile(const QString &fileName)
{
	QFile file(fileName);
	if (!file.open(QFile::ReadOnly | QFile::Text)) {
		QMessageBox::warning(this, tr("TikZ editor"),
		                     tr("Cannot read file %1:\n%2.")
		                     .arg(fileName)
		                     .arg(file.errorString()));
		return;
	}

	QTextStream in(&file);
	QApplication::setOverrideCursor(Qt::WaitCursor);
	textEdit->setPlainText(in.readAll());
	QApplication::restoreOverrideCursor();

	lastDocument = fileName;
	setCurrentFile(fileName);
	addToRecentFilesList(fileName);
	statusBar()->showMessage(tr("File loaded"), 2000);
}

bool ktikz::saveFile(const QString &fileName)
{
	QFile file(fileName);
	if (!file.open(QFile::WriteOnly | QFile::Text)) {
		QMessageBox::warning(this, tr("TikZ editor"),
		                     tr("Cannot write file %1:\n%2.")
		                     .arg(fileName)
		                     .arg(file.errorString()));
		return false;
	}

	QTextStream out(&file);
	QApplication::setOverrideCursor(Qt::WaitCursor);
	out << textEdit->toPlainText();
	QApplication::restoreOverrideCursor();

	lastDocument = fileName;
	setCurrentFile(fileName);
	addToRecentFilesList(fileName);
	statusBar()->showMessage(tr("File saved"), 2000);
	return true;
}

void ktikz::openRecentFile()
{
	QAction *Act = qobject_cast<QAction*>(sender());
	if (Act)
		loadFile(Act->data().toString());
}

void ktikz::addToRecentFilesList(const QString &fileName)
{
	if (recentFilesList.contains(fileName))
		recentFilesList.move(recentFilesList.indexOf(fileName), 0);
	else
	{
		if (recentFilesList.count() >= numOfRecentFiles)
			recentFilesList.removeLast();
		recentFilesList.prepend(fileName);
	}
	updateRecentFilesList();
}

void ktikz::createRecentFilesList()
{
	recentFileActions.clear();
	QAction *Act;
	for (int i = 0; i < numOfRecentFiles; ++i)
	{
		Act = new QAction(this);
		Act->setVisible(false);
		connect(Act, SIGNAL(triggered()), this, SLOT(openRecentFile()));
		recentFileActions.append(Act);
	}

	// when the user has decreased the maximum number of recent files, then we must remove the superfluous entries
	while (recentFilesList.size() > numOfRecentFiles)
		recentFilesList.removeLast();

	updateRecentFilesList();

	recentMenu->clear();
	recentMenu->addActions(recentFileActions);
}

void ktikz::updateRecentFilesList()
{
	if (recentFilesList.count() > 0)
		recentMenu->setEnabled(true);
	else
		recentMenu->setEnabled(false);

	for (int i = 0; i < recentFilesList.count(); ++i)
	{
		recentFileActions[i]->setText(recentFilesList.at(i));
		recentFileActions[i]->setData(recentFilesList.at(i));
		recentFileActions[i]->setVisible(true);
	}
	for (int i = recentFilesList.count(); i < numOfRecentFiles; ++i)
		recentFileActions[i]->setVisible(false);
}

void ktikz::setCurrentFile(const QString &fileName)
{
	curFile = fileName;
	textEdit->document()->setModified(false);
	setWindowModified(false);

	QString shownName;
	if (curFile.isEmpty())
		shownName = "untitled.txt";
	else
		shownName = strippedName(curFile);

	setWindowTitle(tr("%1[*] - %2").arg(shownName).arg(tr("TikZ editor")));
}

QString ktikz::strippedName(const QString &fullFileName)
{
	return QFileInfo(fullFileName).fileName();
}

void ktikz::showCursorPosition()
{
	QTextCursor cur = textEdit->textCursor();
	int curPos = cur.position();
	int startPos;
	cur.movePosition(QTextCursor::StartOfBlock, QTextCursor::MoveAnchor);
	startPos = cur.position();
	positionLabel->setText(tr("Line:") + " " + QString::number(cur.blockNumber() + 1) + "\t" + tr("Col:") + " " + QString::number(curPos - startPos + 1));
}

void ktikz::editComment()
{
	bool go = true;
	QTextCursor cur = textEdit->textCursor();
	if (cur.hasSelection())
	{
		int start = cur.selectionStart();
		int end = cur.selectionEnd();
		cur.setPosition(start, QTextCursor::MoveAnchor);
		cur.movePosition(QTextCursor::StartOfBlock, QTextCursor::MoveAnchor);
		while (cur.position() < end && go)
		{
			cur.insertText("% ");
			++end;
			++end;
			go = cur.movePosition(QTextCursor::NextBlock, QTextCursor::MoveAnchor);
		}
	}
	else
	{
		cur.movePosition(QTextCursor::StartOfBlock, QTextCursor::MoveAnchor);
		cur.insertText("% ");
	}
}

void ktikz::editUncomment()
{
	bool go = true;
	QTextCursor cur = textEdit->textCursor();
	if (cur.hasSelection())
	{
		int start = cur.selectionStart();
		int end = cur.selectionEnd();
		cur.setPosition(start, QTextCursor::MoveAnchor);
		cur.movePosition(QTextCursor::StartOfBlock, QTextCursor::MoveAnchor);
		while (cur.position() < end && go)
		{
			cur.movePosition(QTextCursor::Right, QTextCursor::KeepAnchor, 2);
			if (cur.selectedText() == "% ")
			{
				cur.removeSelectedText();
				--end;
			}
			go = cur.movePosition(QTextCursor::NextBlock, QTextCursor::MoveAnchor);
		}
	}
	else
	{
		cur.movePosition(QTextCursor::StartOfBlock, QTextCursor::MoveAnchor);
		cur.movePosition(QTextCursor::Right, QTextCursor::KeepAnchor, 2);
		if (cur.selectedText() == "% ")
			cur.removeSelectedText();
	}
}
