/***************************************************************************
 *   Copyright (C) 2007 by Florian Hackenberger                            *
 *   Copyright (C) 2007 by Glad Deschrijver                                *
 *   florian@hackenberger.at                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QApplication>
#include <QDebug>
#include <QDir>
#include <QPixmap>
#include <QProcess>
#include <QTextEdit>
#include <QTextStream>
#include "tikzpngpreviewer.h"

TikzPngPreviewer::TikzPngPreviewer(const QTextEdit* tikzTextEdit)
    : minUpdateInterval(0, 0, 1 /*sec*/, 0), tikzTempFile(QDir::tempPath() + "/ktikz/ktikzXXXXXX.tex")
{
	this->tikzTextEdit = tikzTextEdit;
	tikzPdfDoc = 0;
	updateScheduled = false;
	tikzTempFile.open();
	tikzTempFile.setAutoRemove(false);
	runFailed = false;
	keepRunning = true;
	process = 0;
	latexCommand = "pdflatex";
	pdftopsCommand = "pdftops";
	start();
}

TikzPngPreviewer::~TikzPngPreviewer()
{
	// stop the thread before destroying this object
	keepRunning = false;
	updateRequested.wakeAll();
	wait();

	// removing all temporary files
	qDebug() << "removing tempfiles";
	if (!cleanUp())
		qCritical() << "Error: removing tempfile failed";
	wait();
}

void TikzPngPreviewer::setLatexCommand(const QString &command)
{
	latexCommand = command;
}

void TikzPngPreviewer::setPdftopsCommand(const QString &command)
{
	pdftopsCommand = command;
}

Poppler::Document *TikzPngPreviewer::getPdfDocument()
{
	return tikzPdfDoc;
}

QString TikzPngPreviewer::getShortLogText() {
	QMutexLocker lock(&memberLock);
	return shortLogText;
}

QString TikzPngPreviewer::getLogText() {
	QMutexLocker lock(&memberLock);
	return logText;
}

QString TikzPngPreviewer::getParsedLogText(QTextStream *logStream)
{
	QString logText;

	QList<QRegExp> keywordPatterns;
	keywordPatterns << QRegExp("(\\S*):(\\d+): (.*$)")
	    << QRegExp("Undefined control sequence")
	    << QRegExp("LaTeX Warning:") << QRegExp("LaTeX Error:")
	    << QRegExp("Runaway argument?")
	    << QRegExp("Missing character: .*!") << QRegExp("Error:");

	QString logLine;
	while (!logStream->atEnd())
	{
		logLine = logStream->readLine();
		if (keywordPatterns.at(0).indexIn(logLine) > -1)
		{
			// show error message and correct line number
			QString lineNum = QString::number(keywordPatterns[0].cap(2).toInt() - 7);
			QString errorMsg = keywordPatterns[0].cap(3);
			logText += "[LaTeX] Line " + lineNum + ": " + errorMsg;

			// while we don't get a line starting with "l.<number> ...", we have to add the line to the first error message
			QRegExp rx("^l\\.(\\d+)(.*)");
			logLine = logStream->readLine();
			while (rx.indexIn(logLine) < 0 && !logStream->atEnd())
			{
				if (logLine.isEmpty())
					logText += "\n[LaTeX] Line " + lineNum + ": ";
				if (!logLine.startsWith("Type")) // don't add lines that invite the user to type a command, since we are not in the console
					logText += logLine;
				logLine = logStream->readLine();
			}
			logText += "\n";
			if (logStream->atEnd()) break;

			// add the line starting with "l.<number> ..." and the next line
			lineNum = QString::number(rx.cap(1).toInt() - 7);
			logLine = "l." + lineNum + rx.cap(2);
			logText += logLine + "\n";
			logText += logStream->readLine() + "\n";
		}
		else
		{
			for (int i = 1; i < keywordPatterns.size(); ++i)
			{
				if (logLine.contains(keywordPatterns.at(i)))
				{
					logText += logLine + "\n";
					logText += logStream->readLine() + "\n";
					logText += logStream->readLine() + "\n";
					break;
				}
			}
		}
	}

	return logText;
}

bool TikzPngPreviewer::hasRunFailed() {
	QMutexLocker lock(&memberLock);
	return runFailed;
}

void TikzPngPreviewer::run() {
	while(keepRunning) {
		memberLock.lock();
		if(updateScheduled) { // We shall update the picture
			long int msecUpdateInterval = minUpdateInterval.minute()*60000 + minUpdateInterval.second()*1000; // We ignore hours
			if(updateTimer.isValid() && updateTimer.elapsed() < msecUpdateInterval) {
				// Our minUpdateInterval has not elapsed yet, sleep
				int msecsToSleep = msecUpdateInterval - updateTimer.elapsed();
				memberLock.unlock();
				msleep(msecsToSleep);
				continue;
			}
			runFailed = false;
			updateScheduled = false;
			createTempTikzFile();
			memberLock.unlock();
			logText = "";
			QFileInfo tikzTempFileInfo(tikzTempFile);
			if(generatePdfFile()) {
				QFileInfo tikzPdfFileInfo(tikzTempFileInfo.absolutePath(), tikzTempFileInfo.completeBaseName() + ".pdf");
				if(!tikzPdfFileInfo.exists()) {
					qWarning() << "Error:" << qPrintable(tikzPdfFileInfo.absoluteFilePath()) << "does not exists";
				}else {
					// Update widget
					tikzPdfDoc = Poppler::Document::load(tikzPdfFileInfo.absoluteFilePath());
					if(tikzPdfDoc) {
						shortLogText = "[LaTeX] " + tr("Process finished successfully.");
						emit pixmapUpdated();
					}else {
						shortLogText = "[LaTeX] " + tr("Error: loading PDF failed, the file is probably corrupted.");
						qWarning() << "Error: loading PDF failed, the file is probably corrupted:" << qPrintable(tikzPdfFileInfo.absoluteFilePath());
					}
				}
			}
			// parse contents of the log file
			QFileInfo latexLogFileInfo = QFileInfo(tikzTempFileInfo.absolutePath(), tikzTempFileInfo.completeBaseName() + ".log");
			QFile latexLogFile(latexLogFileInfo.absoluteFilePath());
			if (!latexLogFile.open(QFile::ReadOnly))
			{
				shortLogText += "\n[LaTeX] " + tr("Warning: could not load LaTeX logfile.");
				qWarning() << "Warning: could not load latex logfile:" << qPrintable(latexLogFileInfo.absoluteFilePath());
			}
			else
			{
				memberLock.lock();
				QTextStream latexLog(&latexLogFile);
				if (runFailed && !shortLogText.contains("Process aborted"))
					shortLogText = getParsedLogText(&latexLog);
				latexLog.seek(0);
				logText += latexLog.readAll();
				latexLogFile.close();
				memberLock.unlock();
			}
			emit shortLogUpdated();
			emit logUpdated();
			memberLock.lock();
		}else {
			updateRequested.wait(&memberLock);
		}
		memberLock.unlock();
	}
}

void TikzPngPreviewer::createTempTikzFile() {
	if(tikzTempFile.open()) {
		if (tikzString.isEmpty()) // avoid that the previous picture is still displayed
			tikzString = "\\begin{tikzpicture}\n\\end{tikzpicture}";
		QTextStream tikzStream(&tikzTempFile);
		tikzStream << "\\documentclass[12pt]{article}" << endl
		           << "\\usepackage{tikz}" << endl
			   << "\\usepackage{pgf}" << endl
		           << "\\usepackage[active,pdftex,tightpage]{preview}" << endl
		           << "\\PreviewEnvironment[]{tikzpicture}" << endl
		           << "\\begin{document}" << endl
		           << tikzString.toLatin1() << endl
		           << "\\end{document}" << endl;
		tikzTempFile.close();
		qDebug() << "tikz code written to:" << qPrintable(QFileInfo(tikzTempFile).absoluteFilePath());
		shortLogText = "[LaTeX] " + tr("Running...");
		emit shortLogUpdated();
	}
}

bool TikzPngPreviewer::runProcess(const QString &name, const QString &command, const QStringList &arguments) {
	QApplication::setOverrideCursor(Qt::WaitCursor);

	process = new QProcess;
	processAborted = false;
	process->start(command, arguments);
	emit processRunning(true);
	qDebug() << "starting" << command + " " + arguments.join(" ");
	QByteArray buffer;
	QTextStream log(&buffer);
	while (process->state() != QProcess::NotRunning) {
		// still running or starting
		if(process->bytesAvailable()) {
			log << process->readLine(process->bytesAvailable());
		} else {
			process->waitForFinished(100 /*msec*/);
		}
	}
	// Process finished
	emit processRunning(false);
	if (process->bytesAvailable()) {
			log << process->readLine(process->bytesAvailable());
	}
	log.seek(0);
	if (processAborted) {
		shortLogText = "[" + name + "] " + tr("Process aborted.");
		runFailed = true;
	}
	else if (process->exitCode() == 0) {
		shortLogText = "[" + name + "] " + tr("Process finished successfully.");
		runFailed = false;
	} else {
		shortLogText = "[" + name + "] " + tr("Error: run failed.");
		qWarning() << "Error:" << qPrintable(command) << "run failed with exit code:" << process->exitCode();
		memberLock.lock();
		runFailed = true;
		logText = log.readAll();
		memberLock.unlock();
	}
	delete process;
	process = 0;
	QApplication::restoreOverrideCursor();
	emit shortLogUpdated();
	return !runFailed;
}

void TikzPngPreviewer::abortProcess()
{
	if (process)
	{
		process->kill();
		processAborted = true;
	}
}

bool TikzPngPreviewer::generateEpsFile()
{
	QFileInfo tikzTempFileInfo = QFileInfo(tikzTempFile);
	QString fileNameBase = tikzTempFileInfo.absolutePath() + "/" + tikzTempFileInfo.completeBaseName();
	QStringList pdftopsArguments;
	pdftopsArguments << fileNameBase + ".pdf" << fileNameBase + ".eps";
	return runProcess("pdftops", pdftopsCommand, pdftopsArguments);
}

bool TikzPngPreviewer::generatePdfFile() {
	// remove log file before running pdflatex again
	QFileInfo tikzTempFileInfo = QFileInfo(tikzTempFile);
	QString fileNameBase = tikzTempFileInfo.absolutePath() + "/" + tikzTempFileInfo.completeBaseName();
	QDir::root().remove(fileNameBase + ".log");

	QStringList latexArguments;
	latexArguments << "-halt-on-error" << "-file-line-error" << "-interaction" << "batchmode" << "-output-directory" << tikzTempFileInfo.absolutePath() << tikzTempFileInfo.absoluteFilePath();
	return runProcess("LaTeX", latexCommand, latexArguments);
}

void TikzPngPreviewer::regeneratePreview() {
	QMutexLocker locker(&memberLock);
	tikzString = tikzTextEdit->toPlainText();
	updateScheduled = true;
	updateTimer.start();
	updateRequested.wakeAll();
}

bool TikzPngPreviewer::exportImage(const QString &fileName, const QString &type)
{
	QString fileName2 = fileName;
	if (!fileName.endsWith(type))
		fileName2 = fileName + "." + type;

	QFileInfo tikzTempFileInfo(tikzTempFile);
	QFileInfo tikzPdfFileInfo = QFileInfo(tikzTempFileInfo.absolutePath(), tikzTempFileInfo.completeBaseName() + ".pdf");

	if (type == "eps")
	{
		if (!generateEpsFile()) return false;
		QFileInfo tikzEpsFileInfo = QFileInfo(tikzTempFileInfo.absolutePath(), tikzTempFileInfo.completeBaseName() + ".eps");
		QFile::remove(fileName2);
		return QFile::copy(tikzEpsFileInfo.absoluteFilePath(), fileName2);
	}
	else if (type == "pdf")
	{
		QFile::remove(fileName2);
		return QFile::copy(tikzPdfFileInfo.absoluteFilePath(), fileName2);
	}
	return false;
}

bool TikzPngPreviewer::cleanUp()
{
	bool success = true;

	QFileInfo tikzTempFileInfo(tikzTempFile);
	QDir tikzTempDir(tikzTempFileInfo.absolutePath());
	QStringList filters;
	filters << tikzTempFileInfo.completeBaseName() + ".*";

	QString fileName;
	foreach (fileName, tikzTempDir.entryList(filters))
		success = success && tikzTempDir.remove(fileName);
	return success;
}
