/***************************************************************************
 *   Copyright (C) 2007 by Glad Deschrijver                                *
 *   Glad.Deschrijver@UGent.be                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <QAction>
#include <QComboBox>
#include <QContextMenuEvent>
#include <QGraphicsPixmapItem>
#include <QLineEdit>
#include <QMenu>
#include <QToolBar>
#include "tikzpreview.h"

TikzPreview::TikzPreview(QWidget *parent)
    : QGraphicsView(parent)
{
	tikzImage = new QPixmap();
	tikzScene = new QGraphicsScene;
	tikzPixmapItem = tikzScene->addPixmap(*tikzImage);
	setScene(tikzScene);
	setDragMode(QGraphicsView::ScrollHandDrag);
	setWhatsThis("<p>" + tr("Here the preview image of "
	    "your TikZ code is shown.  You can zoom in and out, and you "
	    "can scroll the image by dragging it.") + "</p>");

	createActions();
	createViewToolBar();

	currentPage = 0;
	zoomFactor = 1;
	minZoomFactor = 0.1;
	maxZoomFactor = 6;
}

void TikzPreview::contextMenuEvent(QContextMenuEvent *event)
{
	QMenu *menu = new QMenu(this);
	menu->addActions(getActions());
	menu->exec(event->globalPos());
	delete menu;
}

QSize TikzPreview::sizeHint() const
{
	QSize size(200, 200);
	return size;
}

void TikzPreview::setZoomFactor()
{
	QString newZoomFactorText = zoomCombo->lineEdit()->text().remove(QRegExp("[^\\d\\.]*"));
	zoomFactor = newZoomFactorText.toDouble() / 100;

	if (zoomFactor < minZoomFactor)
		zoomFactor = minZoomFactor;
	else if (zoomFactor > maxZoomFactor)
		zoomFactor = maxZoomFactor;

	showPdfPage();
	setSceneRect(tikzScene->itemsBoundingRect());
}

void TikzPreview::zoomIn()
{
	if (zoomFactor < 0.999999)
		zoomFactor += 0.1;
	else if (zoomFactor < 1.999999)
		zoomFactor += 0.2;
	else
		zoomFactor += 0.5;
	if (zoomFactor > maxZoomFactor)
		zoomFactor = maxZoomFactor;
	showPdfPage();
	setSceneRect(tikzScene->itemsBoundingRect());
}

void TikzPreview::zoomOut()
{
	if (zoomFactor < 1.000001)
		zoomFactor -= 0.1;
	else if (zoomFactor < 2.000001)
		zoomFactor -= 0.2;
	else
		zoomFactor -= 0.5;
	if (zoomFactor < minZoomFactor)
		zoomFactor = minZoomFactor;
	showPdfPage();
	setSceneRect(tikzScene->itemsBoundingRect());
}

void TikzPreview::showPreviousPage()
{
	if (currentPage > 0)
		--currentPage;
	showPdfPage();
}

void TikzPreview::showNextPage()
{
	if (currentPage < tikzPdfDoc->numPages() - 1)
		++currentPage;
	showPdfPage();
}

void TikzPreview::showPdfPage()
{
	zoomCombo->lineEdit()->setText(QString::number(zoomFactor * 100) + "%");

	if (!tikzPdfDoc) return;

	Poppler::Page *pdfPage = tikzPdfDoc->page(currentPage);
//	pdfPage->renderToPixmap(tikzImage); // render pdf using Qt4's Arthur renderer; currently this gives ugly results, therefore we use the Splash renderer
	tikzImage = pdfPage->splashRenderToPixmap(zoomFactor *72, zoomFactor *72);
	tikzPixmapItem->setPixmap(*tikzImage);
	tikzPixmapItem->update();
}

void TikzPreview::pixmapUpdated(Poppler::Document *tikzPdfDoc)
{
	this->tikzPdfDoc = tikzPdfDoc;
	int numOfPages = tikzPdfDoc->numPages();

	int sepPos = viewToolBar->actions().indexOf(previousPageAct) - 1;
	bool visible = false;
	if (numOfPages > 1)
		visible = true;
	viewToolBar->actions().at(sepPos)->setVisible(visible);
	previousPageAct->setVisible(visible);
	nextPageAct->setVisible(visible);

	if (currentPage >= numOfPages)
		currentPage = 0;

	showPdfPage();
}

void TikzPreview::createActions()
{
	zoomInAct = new QAction(QIcon(":/images/zoom-in.png"), tr("Zoom &In"), this);
	zoomInAct->setShortcut(QKeySequence::ZoomIn);
	zoomInAct->setStatusTip(tr("Zoom preview in"));
	zoomInAct->setWhatsThis("<p>" + tr("Zoom preview in by a predetermined factor.") + "</p>");
	connect(zoomInAct, SIGNAL(triggered()), this, SLOT(zoomIn()));

	zoomOutAct = new QAction(QIcon(":/images/zoom-out.png"), tr("Zoom &Out"), this);
	zoomOutAct->setShortcut(QKeySequence::ZoomOut);
	zoomOutAct->setStatusTip(tr("Zoom preview out"));
	zoomOutAct->setWhatsThis("<p>" + tr("Zoom preview out by a predetermined factor.") + "</p>");
	connect(zoomOutAct, SIGNAL(triggered()), this, SLOT(zoomOut()));

	previousPageAct = new QAction(QIcon(":/images/go-previous.png"), tr("&Previous image"), this);
	previousPageAct->setShortcut(tr("Alt+Left", "Go to previous page"));
	previousPageAct->setStatusTip(tr("Show previous image in preview"));
	previousPageAct->setWhatsThis("<p>" + tr("Show the preview of the previous tikzpicture in the TikZ code.") + "</p>");
	connect(previousPageAct, SIGNAL(triggered()), this, SLOT(showPreviousPage()));

	nextPageAct = new QAction(QIcon(":/images/go-next.png"), tr("&Next image"), this);
	nextPageAct->setShortcut(tr("Alt+Right", "Go to next page"));
	nextPageAct->setStatusTip(tr("Show next image in preview"));
	nextPageAct->setWhatsThis("<p>" + tr("Show the preview of the next tikzpicture in the TikZ code.") + "</p>");
	connect(nextPageAct, SIGNAL(triggered()), this, SLOT(showNextPage()));

	previousPageAct->setVisible(false);
	nextPageAct->setVisible(false);
}

QList<QAction*> TikzPreview::getActions()
{
	QList<QAction*> actions;
	actions << zoomInAct << zoomOutAct;
	QAction *Act = new QAction(this);
	Act->setSeparator(true);
	actions << Act;
	actions << previousPageAct << nextPageAct;
	return actions;
}

void TikzPreview::createViewToolBar()
{
	QStringList zoomSizesList;
	zoomSizesList << "12.50%" << "25%" << "50%" << "75%" << "100%" << "125%" << "150%" << "200%" << "250%" << "300%";
	zoomCombo = new QComboBox;
	zoomCombo->setEditable(true);
	zoomCombo->setToolTip(tr("Select or insert zoom factor here"));
	zoomCombo->setWhatsThis("<p>" + tr("Select the zoom factor here.  "
	    "Alternatively, you can also introduce a zoom factor and "
	    "press Enter.") + "</p>");
	zoomCombo->insertItems(0, zoomSizesList);
	zoomCombo->setCurrentIndex(zoomCombo->findText("100%"));
	connect(zoomCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(setZoomFactor()));
	connect(zoomCombo->lineEdit(), SIGNAL(returnPressed()), this, SLOT(setZoomFactor()));

	viewToolBar = new QToolBar(tr("View"));
	viewToolBar->setObjectName("ViewToolBar");
	viewToolBar->addAction(zoomInAct);
	viewToolBar->addWidget(zoomCombo);
	viewToolBar->addAction(zoomOutAct);
	viewToolBar->addSeparator();
	viewToolBar->addAction(previousPageAct);
	viewToolBar->addAction(nextPageAct);
	int sepPos = viewToolBar->actions().indexOf(previousPageAct) - 1;
	viewToolBar->actions().at(sepPos)->setVisible(false);
}

QToolBar *TikzPreview::getViewToolBar()
{
	return viewToolBar;
}

QPixmap *TikzPreview::getPixmap()
{
	return tikzImage;
}
